const http = require('http');
const { Semantics, Semantic } = require('./src/semantics');
const { Generators, Generator } = require('./src/generators');
const { Config } = require('./src/config');
const fetch = require('node-fetch')

const process = (url, key, config, retries = 2) => {
  let data = Object.assign( { key, version: '3' }, config.config );
  const generators = new Generators(data.generators.map( (g) => new Generator(g[0], g[1]) ) )
  delete data['generators']
  const semantics = new Semantics(data.semantics.map( (g) => new Semantic(g[0], g[1]) ) )
  delete data['semantics']

  const doWithRetries = (n) => {
    return fetch(`${url}/process`, {
      method: "POST",
      body: JSON.stringify(data),
      timeout: 1000 * 60 * 5, // it does not respect this timeout so that's why I have the retries
      headers: {
        mode: "no-cors",
        "Content-Type": "application/json"
      },
      }).then( result => {
        if (result.ok) {
          return result.json()
        }
        if (result.status == 504){
          if (n == 0) {
            throw `Error ${result.status} - ${result.statusText}`;
          } else {
            return doWithRetries(n-1)
          }
        }
        throw `Error ${result.status} - ${result.statusText}`;
      })
    }
      
  return doWithRetries(retries) 
    .then( json => {
      if (json.status != 200) {
        throw `Error ${json.status} - ${json.error}`;
        return;
      }
      if (json.results) {
        json.results = json.results.map( (result) => semantics.apply(config.config.objects, result) )
        json.generated = json.results.map( (result) => generators.apply(result) )
      }
      return json;
    });
}

module.exports = { process }
